@@ -12,6 +12,7 @@
 
 import numpy as np
 from scipy.special import gamma
+from scipy.optimize import minimize_scalar
 
 from ....utilities import cosd, sind, tand
 from .gaussian_model_base import GaussianModel
@@ -22,7 +23,9 @@ class Blondel(GaussianModel):
     """
     Blondel is a direct implementation of the super-Gaussian model
     described in :cite:`bcv-blondel2020alternative` with GCH disabled by
-    default. See :cite:`bcv-King2019Controls` for info on GCH.
+    default. See :cite:`bcv-King2019Controls` for info on GCH. The current
+    implementation is based on the calibration proposed in
+    :cite:`bcv-Cathelain2020calibration`
 
     References:
         .. bibliography:: /source/zrefs.bib
@@ -32,12 +35,14 @@ class Blondel(GaussianModel):
     """
 
     default_parameters = {
-        "a_s": 0.3837,
-        "b_s": 0.003678,
-        "c_s": 0.2,
-        "a_f": 3.11,
-        "b_f": -0.68,
-        "c_f": 2.41,
+        "a_s": 0.179367259,
+        "b_s": 0.0118889215,
+        "c_s1": 0.0563691592,
+        "c_s2": 0.13290157,
+        "b_f1": 1.59,
+        "b_f2": -23.3114654,
+        "b_f3": -2.15199155,
+        "c_f": 2.98262872,
         "calculate_VW_velocities": False,
         "use_yaw_added_recovery": False,
         "eps_gain": 0.2,
@@ -58,12 +63,17 @@ class Blondel(GaussianModel):
                     -   **b_s**: Parameter used to determine the linear
                         relationship between the turbulence intensity and the
                         width of the Gaussian wake shape.
-                    -   **c_s**: Parameter used to determine the linear
+                    -   **c_s1**: Parameter used to determine the linear
                         relationship between the turbulence intensity and the
                         width of the Gaussian wake shape.
-                    -   **a_f**: Parameter used to determine super-Gaussian
+                    -   **c_s2**: Parameter used to determine the linear
+                        relationship between the turbulence intensity and the
+                        width of the Gaussian wake shape.
+                    -   **b_f1**: Parameter used to determine super-Gaussian
                         order.
-                    -   **b_f**: Parameter used to determine super-Gaussian
+                    -   **b_f2**: Parameter used to determine super-Gaussian
+                        order.
+                    -   **b_f3**: Parameter used to determine super-Gaussian
                         order.
                     -   **c_f**: Parameter used to determine super-Gaussian
                         order.
@@ -83,15 +93,17 @@ class Blondel(GaussianModel):
         model_dictionary = self._get_model_dict(__class__.default_parameters)
 
         # wake expansion parameters
-        # Table 2 of reference in docstring
+        # Table 4 of reference [3] in docstring
         self.a_s = model_dictionary["a_s"]
         self.b_s = model_dictionary["b_s"]
-        self.c_s = model_dictionary["c_s"]
+        self.c_s1 = model_dictionary["c_s1"]
+        self.c_s2 = model_dictionary["c_s2"]
 
         # fitted parameters for super-Gaussian order n
-        # Table 3 of reference in docstring
-        self.a_f = model_dictionary["a_f"]
-        self.b_f = model_dictionary["b_f"]
+        # Table 4 of reference [3] in docstring
+        self.b_f1 = model_dictionary["b_f1"]
+        self.b_f2 = model_dictionary["b_f2"]
+        self.b_f3 = model_dictionary["b_f3"]
         self.c_f = model_dictionary["c_f"]
 
         self.model_grid_resolution = None
@@ -167,7 +179,8 @@ class Blondel(GaussianModel):
         xR = yR * tand(yaw) + turbine_coord.x1
 
         # Compute scaled variables (Eq 1, pp 3 of ref. [1] in docstring)
-        x_tilde = (x_locations - turbine_coord.x1) / D
+        # Use absolute value to avoid overflows
+        x_tilde = np.abs(x_locations - turbine_coord.x1) / D
         r_tilde = (
             np.sqrt(
                 (y_locations - turbine_coord.x2 - delta) ** 2 + (z_locations - HH) ** 2,
@@ -176,14 +189,24 @@ class Blondel(GaussianModel):
             / D
         )
 
-        # Calculate Beta (Eq 10, pp 5 of ref. [1] in docstring)
-        beta = 0.5 * ((1 + np.sqrt(1 - Ct)) / np.sqrt(1 - Ct))
+        # Calculate Beta (Eq 10, pp 5 of ref. [1] and table 4 of ref. [2] in docstring)
+        beta = 0.5 * (1.0 + np.sqrt(1.0 - Ct)) / np.sqrt(1.0 - Ct)
+        k = self.a_s * TI + self.b_s
+        eps = (self.c_s1 * Ct + self.c_s2) * np.sqrt(beta)
 
-        # Calculate sigma_tilde (Eq 9, pp 5 of ref. [1] in docstring)
-        sigma_tilde = (self.a_s * TI + self.b_s) * x_tilde + self.c_s * np.sqrt(beta)
+        # Calculate sigma_tilde (Eq 9, pp 5 of ref. [1] and table 4 of ref. [2] in docstring)
+        sigma_tilde = k * x_tilde + eps
 
-        # Calculate n (Eq 13, pp 6 of ref. [1] in docstring)
-        n = self.a_f * np.exp(self.b_f * x_tilde) + self.c_f
+        # Calculate super-Gaussian order using iterative method
+        root_n = minimize_scalar(
+            self.match_AD_theory,
+            bounds=(0.0, 10.0),
+            method="bounded",
+            args=(Ct, eps, yaw, self.c_f),
+        )
+        a_f = root_n.x
+        b_f = self.b_f1 * np.exp(self.b_f2 * TI) + self.b_f3
+        n = a_f * np.exp(b_f * x_tilde) + self.c_f
 
         # Calculate max vel def (Eq 5, pp 4 of ref. [1] in docstring)
         a1 = 2 ** (2 / n - 1)
@@ -212,11 +235,43 @@ class Blondel(GaussianModel):
             np.zeros(np.shape(velDef1)),
         )
 
+    def match_AD_theory(self, n_af, Ct, eps, yaw, n_min):
+        """
+        Calculate the difference in velocities at the rotor plane using the
+        Actuator Disk theory and the super-Gaussian model.
+
+        Args:
+            n_af (float): Current value of the a_f parameter (.).
+            Ct (float): Rotor thrust coefficient (.).
+            eps (float): Initial wake expansion parameter (.)
+            yaw (float): Rotor yaw angle (deg).
+            n_min (float): Minimum value of the super-Gaussian order (.).
+
+        Returns:
+            float: difference betweend computed velocity deficit at the rotor
+            disk and induction (AD theory)
+        """
+
+        # Calculate n using af and minimum value c_f  (Eq 13, pp 6 of ref. [1] in docstring)
+        n_val = n_af + n_min
+        induction = 0.5 * (1.0 - np.sqrt(1.0 - Ct))
+
+        # Calculate velocity deficit at the disk (x_tilde = 0)
+        u_disk = np.power(2.0, 2.0 / n_val - 1.0) - np.sqrt(
+            np.power(2.0, 4.0 / n_val - 2.0)
+            - n_val
+            * Ct
+            * cosd(yaw)
+            / (16.0 * np.power(eps, 4.0 / n_val) * gamma(2.0 / n_val))
+        )
+
+        return abs(u_disk - induction)
+
     @property
     def a_s(self):
         """
         Constant coefficient used in calculation of wake expansion. See
-        Eqn. 9 in [1].
+        Eqn. 9 in [1] and Table 4 in [3].
 
         **Note:** This is a virtual property used to "get" or "set" a value.
 
@@ -252,7 +307,7 @@ class Blondel(GaussianModel):
     def b_s(self):
         """
         Constant coefficient used in calculation of wake expansion. See
-        Eqn. 9 in [1].
+        Eqn. 9 in [1] and Table 4 in [3].
 
         **Note:** This is a virtual property used to "get" or "set" a value.
 
@@ -285,10 +340,10 @@ class Blondel(GaussianModel):
             )
 
     @property
-    def c_s(self):
+    def c_s1(self):
         """
         Linear constant used in calculation of wake expansion. See
-        Eqn. 9 in [1].
+        Eqn. 9 in [1] and Table 4 in [3].
 
         **Note:** This is a virtual property used to "get" or "set" a value.
 
@@ -301,30 +356,30 @@ class Blondel(GaussianModel):
         Raises:
             ValueError: Invalid value.
         """
-        return self._c_s
+        return self._c_s1
 
-    @c_s.setter
-    def c_s(self, value):
+    @c_s1.setter
+    def c_s1(self, value):
         if type(value) is not float:
             err_msg = (
-                "Invalid value type given for c_s: {}, " + "expected float."
+                "Invalid value type given for c_s1: {}, " + "expected float."
             ).format(value)
             self.logger.error(err_msg, stack_info=True)
             raise ValueError(err_msg)
-        self._c_s = value
-        if value != __class__.default_parameters["c_s"]:
+        self._c_s1 = value
+        if value != __class__.default_parameters["c_s1"]:
             self.logger.info(
                 (
-                    "Current value of c_s, {0}, is not equal to tuned "
+                    "Current value of c_s1, {0}, is not equal to tuned "
                     + "value of {1}."
-                ).format(value, __class__.default_parameters["c_s"])
+                ).format(value, __class__.default_parameters["c_s1"])
             )
 
     @property
-    def a_f(self):
+    def c_s2(self):
         """
-        Constant exponent coefficient used in calculation of the super-Gaussian
-        order. See Eqn. 13 in [1].
+        Linear constant used in calculation of wake expansion. See
+        Eqn. 9 in [1] and Table 4 in [3].
 
         **Note:** This is a virtual property used to "get" or "set" a value.
 
@@ -337,30 +392,30 @@ class Blondel(GaussianModel):
         Raises:
             ValueError: Invalid value.
         """
-        return self._a_f
+        return self._c_s2
 
-    @a_f.setter
-    def a_f(self, value):
+    @c_s2.setter
+    def c_s2(self, value):
         if type(value) is not float:
             err_msg = (
-                "Invalid value type given for a_f: {}, " + "expected float."
+                "Invalid value type given for c_s2: {}, " + "expected float."
             ).format(value)
             self.logger.error(err_msg, stack_info=True)
             raise ValueError(err_msg)
-        self._a_f = value
-        if value != __class__.default_parameters["a_f"]:
+        self._c_s2 = value
+        if value != __class__.default_parameters["c_s2"]:
             self.logger.info(
                 (
-                    "Current value of a_f, {0}, is not equal to tuned "
+                    "Current value of c_s2, {0}, is not equal to tuned "
                     + "value of {1}."
-                ).format(value, __class__.default_parameters["a_f"])
+                ).format(value, __class__.default_parameters["c_s2"])
             )
 
     @property
-    def b_f(self):
+    def b_f1(self):
         """
         Constant exponent coefficient used in calculation of the super-Gaussian
-        order. See Eqn. 13 in [1].
+        order. See Eqn. 13 in [1] and Table 4 in [3].
 
         **Note:** This is a virtual property used to "get" or "set" a value.
 
@@ -373,30 +428,102 @@ class Blondel(GaussianModel):
         Raises:
             ValueError: Invalid value.
         """
-        return self._b_f
+        return self._b_f1
 
-    @b_f.setter
-    def b_f(self, value):
+    @b_f1.setter
+    def b_f1(self, value):
         if type(value) is not float:
             err_msg = (
-                "Invalid value type given for b_f: {}, " + "expected float."
+                "Invalid value type given for b_f1: {}, " + "expected float."
             ).format(value)
             self.logger.error(err_msg, stack_info=True)
             raise ValueError(err_msg)
-        self._b_f = value
-        if value != __class__.default_parameters["b_f"]:
+        self._b_f1 = value
+        if value != __class__.default_parameters["b_f1"]:
             self.logger.info(
                 (
-                    "Current value of b_f, {0}, is not equal to tuned "
+                    "Current value of b_f1, {0}, is not equal to tuned "
                     + "value of {1}."
-                ).format(value, __class__.default_parameters["b_f"])
+                ).format(value, __class__.default_parameters["b_f1"])
+            )
+
+    @property
+    def b_f2(self):
+        """
+        Constant exponent coefficient used in calculation of the super-Gaussian
+        order. See Eqn. 13 in [1] and Table 4 in [3].
+
+        **Note:** This is a virtual property used to "get" or "set" a value.
+
+        Args:
+            value (float): Value to set.
+
+        Returns:
+            float: Value currently set.
+
+        Raises:
+            ValueError: Invalid value.
+        """
+        return self._b_f2
+
+    @b_f2.setter
+    def b_f2(self, value):
+        if type(value) is not float:
+            err_msg = (
+                "Invalid value type given for b_f2: {}, " + "expected float."
+            ).format(value)
+            self.logger.error(err_msg, stack_info=True)
+            raise ValueError(err_msg)
+        self._b_f2 = value
+        if value != __class__.default_parameters["b_f2"]:
+            self.logger.info(
+                (
+                    "Current value of b_f2, {0}, is not equal to tuned "
+                    + "value of {1}."
+                ).format(value, __class__.default_parameters["b_f2"])
+            )
+
+    @property
+    def b_f3(self):
+        """
+        Constant exponent coefficient used in calculation of the super-Gaussian
+        order. See Eqn. 13 in [1] and Table 4 in [3].
+
+        **Note:** This is a virtual property used to "get" or "set" a value.
+
+        Args:
+            value (float): Value to set.
+
+        Returns:
+            float: Value currently set.
+
+        Raises:
+            ValueError: Invalid value.
+        """
+        return self._b_f3
+
+    @b_f3.setter
+    def b_f3(self, value):
+        if type(value) is not float:
+            err_msg = (
+                "Invalid value type given for b_f3: {}, " + "expected float."
+            ).format(value)
+            self.logger.error(err_msg, stack_info=True)
+            raise ValueError(err_msg)
+        self._b_f3 = value
+        if value != __class__.default_parameters["b_f3"]:
+            self.logger.info(
+                (
+                    "Current value of b_f3, {0}, is not equal to tuned "
+                    + "value of {1}."
+                ).format(value, __class__.default_parameters["b_f3"])
             )
 
     @property
     def c_f(self):
         """
         Linear constant used in calculation of the super-Gaussian order. See
-        Eqn. 13 in [1].
+        Eqn. 13 in [1] and Table 4 in [3].
 
         **Note:** This is a virtual property used to "get" or "set" a value.
 
